-----------------------------------------------------------------------------------------------
-- Client Lua Script for StickyWhisper
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------
 
require "Apollo"
 
-----------------------------------------------------------------------------------------------
-- StickyWhisper Module Definition
-----------------------------------------------------------------------------------------------
local StickyWhisper = {} 

StickyWhisper.Settings = {
	Enabled = true
}
 
-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function StickyWhisper:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 
    return o
end

function StickyWhisper:Init()
	local bHasConfigureFunction = true
	local strConfigureButtonText = "Sticky Whisper"
	local tDependencies = nil
	
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
	Apollo.RegisterSlashCommand("sw", "SlashHandler", self);
	
	-- Hook into the chat message system. We'll grab incoming messages, check if they are
	-- whispers, and save the sender. This is then used on input focus if the user is in the
	-- whisper channel to pre-populate the command and recipient
	--Apollo.RegisterEventHandler("ChatMessage", "OnChatMessage", self)
end

function StickyWhisper:SlashHandler()
	self:OnConfigure()
end

-----------------------------------------------------------------------------------------------
-- StickyWhisper OnLoad
-----------------------------------------------------------------------------------------------
function StickyWhisper:OnLoad()
    -- load our form file
	self.xmlDoc = XmlDoc.CreateFromFile("StickyWhisper.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)
end

function StickyWhisper:OnConfigure()
	-- Open the configuration window if it exists and is loaded
	if self.xmlDoc and self.xmlDoc:IsLoaded() then
		self.wndMain:Show(true, true)
	end
end

-----------------------------------------------------------------------------------------------
-- StickyWhisper OnDocLoaded
-----------------------------------------------------------------------------------------------
function StickyWhisper:OnDocLoaded()

	if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
	    self.wndMain = Apollo.LoadForm(self.xmlDoc, "frmSW", nil, self)
		if self.wndMain == nil then
			Apollo.AddAddonErrorText(self, "Could not load the config window for some reason.")
			return
		end

		-- Set the configuration checkbox based on the settings
		StickyWhisper.EnableSWCheckbox = self.wndMain:FindChild("cbxEnableSW")
		StickyWhisper.EnableSWCheckbox:SetCheck(StickyWhisper.Settings.Enabled)
		self.wndMain:Show(false, true)
		
		--StickyWhisper.chatAddon = Apollo.GetAddon("ChatLog")
		--boop = self.wndMain:FindChild("")
		if StickyWhisper.Settings.Enabled then
			self:StickyWhisper()
		end
	end	
end	
	-----------------------------------------------------------------------------------------
-- StickyWhisper Functions
-- !!!! Thanks to Adelea for giving permission to use their implementation !!!!
-----------------------------------------------------------------------------------------------
function StickyWhisper:StickyWhisper()
	local _chataddons = {
			"ChatLog",
			"ChatFixed",
			"ImprovedChatLog",
			"BetterChatLog",
		}
			
		local _chatprefix = {
			["/w"] = true,
			["/t"] = true,
			["/aw"] = true,
		}
		
		local _chatchannels = {
			[ChatSystemLib.ChatChannel_AccountWhisper] = "/aw",
			[ChatSystemLib.ChatChannel_Whisper] = "/w"
		}
	
		for _,v in pairs(_chataddons) do
			StickyWhisper.CL = Apollo.GetAddon(v)
			if StickyWhisper.CL then
				StickyWhisper.OnChatInputReturn = StickyWhisper.CL.OnChatInputReturn
				function StickyWhisper.CL.OnChatInputReturn(s,h,c,t)
					local r = {}
					for token in string.gmatch(t, "[^%s]+") do
						table.insert(r, token)
					end
	
					local tChatData = c:GetParent():GetData()
					local chanCurrent = tChatData.channelCurrent:GetType()
					
					if _chatprefix[r[1]] and r[2] then
						StickyWhisper.tChatPrefix = r[1]
						StickyWhisper.tLastWhisperer = r[2]
					end
				
					if _chatchannels[chanCurrent] and StickyWhisper.tLastWhisperer then
						if string.sub(t,1,1) ~= "/" then
							t = StickyWhisper.tChatPrefix .. " " .. StickyWhisper.tLastWhisperer .. " " .. t
						end
					end
					
					StickyWhisper.OnChatInputReturn(s,h,c,t)
				end
				break
			end
		end
	end

-- Old code
--[[
function StickyWhisper:StickyWhisper()
	if StickyWhisper.chatAddon then
		StickyWhisper.InputGainedFocus = StickyWhisper.chatAddon.OnInputGainedFocus
		function StickyWhisper.chatAddon.OnInputGainedFocus(s, wndHandler, wndControl)
			local wndEdit = StickyWhisper.HelperGetCurrentEditbox(StickyWhisper.chatAddon)
			
			-- We're going to populate that input with the appropriate command
			-- and person if the user's last received message was a whisper
			if wndEdit then
--				-- Get the chat data 
--				local chatData = wndControl:GetParent():GetData()
--				local chatChannel = chatData.channelCurrent
--				local currentChannelName = chatChannel:GetName();
--				Print(inspect(getmetatable(chatChannel)))
--				Print("######################")
--				Print("Name: " .. StickyWhisper.LastSender)
--				Print("ChannelType: " .. StickyWhisper.LastChannelType)
--				Print("WhisperChannel: " .. ChatSystemLib.ChatChannel_Whisper:GetName())
--				Print("Members: " .. inspect(chatChannel:GetMembers()))
				
				
				-- If the last message was a whisper, pre-populate the command
				-- and recipient
				-- Check if we're in the whisper or account whisper channel. If not, we
				-- don't really care
				
				--if StickyWhisper.LastSender ~= "" then
					-- Check if the message whas a whisper. We don't care about other messages
				    if StickyWhisper.LastChannelType == ChatSystemLib.ChatChannel_Whisper then
						--Print("whisper!")
						
						wndEdit:SetText("/w " .. StickyWhisper.LastSender .. " ")
						wndEdit:SetSel(wndEdit:GetText():len(), 0)
					end
				--end
				--wndEdit:SetText(chatData.channelCurrent:GetCommand())
			end
		end
	end
end
--]]

function StickyWhisper:OnChatMessage(channelCurrent, tMessage)
	local channelType = channelCurrent:GetType()
	StickyWhisper.LastChannelType = channelType
	--Print(StickyWhisper.LastChannelType .. "-" .. channelCurrent:GetName())
	-- Check if the message was a whisper. We don't care about other messages
    if channelType == ChatSystemLib.ChatChannel_Whisper then
		StickyWhisper.LastSender = tMessage.strSender
		StickyWhisper.LastSenderRealm = tMessage.strRealmName
		if StickyWhisper.LastSenderRealm ~= "" then
			StickyWhisper.LastSender = StickyWhisper.LastSender .. "@" .. StickyWhisper.LastSenderRealm
		end
	end
end

function StickyWhisper.HelperGetCurrentEditbox()
	local wndEdit
	-- find the last used chat window
	for idx, wndCurrent in pairs(StickyWhisper.chatAddon.tChatWindows) do
		if wndCurrent:FindChild("Input"):GetData() then
			wndEdit = wndCurrent:FindChild("Input")
			break
		end
	end

	-- if none found, use the first on our list
	if wndEdit == nil then
		for idx, wndCurrent in pairs(StickyWhisper.chatAddon.tChatWindows) do
			wndEdit = wndCurrent:FindChild("Input")
			break
		end
	end

	return wndEdit
end

-----------------------------------------------------------------------------------------------
-- StickyWhisperForm Functions
-----------------------------------------------------------------------------------------------
-- when the OK button is clicked
function StickyWhisper:OnOK()
	self.wndMain:Close() -- hide the window
end

-- when the Cancel button is clicked
function StickyWhisper:OnCancel()
	self.wndMain:Close() -- hide the window
end


function StickyWhisper:ToggleStickyWhisper()
	StickyWhisper.Settings.Enabled = StickyWhisper.EnableSWCheckbox.IsChecked()
	
	-- Update things to reflect the new toggle
	if StickyWhisper.Settings.Enabled then
		self:StickyWhisper()
	end
end

function StickyWhisper:OnSave(eType)
	if eType == GameLib.CodeEnumAddonSaveLevel.Character then
		return StickyWhisper.Settings
	end
end

function StickyWhisper:OnRestore(eType, tSavedData)
	if eType == GameLib.CodeEnumAddonSaveLevel.Character then
		for k,v in pairs(tSavedData) do
			StickyWhisper.Settings[k] = v
		end
	end
end

-----------------------------------------------------------------------------------------------
-- StickyWhisper Instance
-----------------------------------------------------------------------------------------------
local StickyWhisperInst = StickyWhisper:new()
StickyWhisperInst:Init()
